from functools import wraps
from sanic import Sanic, exceptions

class SanicApiKey:
    def __init__(self, app=None,
                arg='key',
                header=None,
                form=None,
                keys=None,
                error='Auth required.'
                ):
        self.api_keys = keys
        self.header = header
        self.arg = arg
        self.form = form
        self.error = error

        if app is not None:
            self.init_app(app)

    def init_app(self, app: Sanic):
        """hook on request start etc."""
        app.register_middleware(self.open_session, 'request')

    async def open_session(self, request):
        pass

    async def _is_api_key(self, request):
        """
        check key is valid api key
        """

        if self.header:
            token = request.headers.get(self.header, None)
        elif self.form:
            if self.form in request.form:
                token = request.form[self.form][0]
            else:
                token = None
        else:
            if self.arg in request.args:
                token = request.args[self.arg][0]
            else:
                token = None

        self.token = token
        return token in self.api_keys

    def get_token(self):
        """
        Get logged api key
        """

        return self.token if self.token else ''
    
    def add_key(self, key : str):
        """
        Add API Key.
        """

        self.api_keys.append(key)

    def key_required(self, handler=None):
        """
        wrapper
        """
        
        @wraps(handler)
        async def wrapper(request, *args, **kwargs):
            if not await self._is_api_key(request):
                if hasattr(self.error, '__call__'):
                    return await self.error(request)
                else:
                    raise exceptions.Unauthorized(self.error)
            return await handler(request, *args, **kwargs)
        return wrapper



from setuptools import setup
import sys

required = ['sanic']
long_description = ""
with open('README.md') as f:
    long_description += f.read()

setup(
    name='SanicApiKey',
    version='1.0.1',
    description='Basic Sanic API Key plugin.',
    long_description=long_description,
    author='Yusuf Usta',
    author_email='yusuf@fusuf.codes',
    maintainer='Yusuf Usta',
    maintainer_email='yusuf@fusuf.codes',
    url='https://github.com/quiec/SanicApiKey',
    license='GPL3',
    packages=['SanicApiKey'],
    install_requires=required,
    keywords=['sanic'],
    long_description_content_type="text/markdown",
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ],
)